import customtkinter as ctk
from qrImageIndexerGUI import generate_qr_window
from qrImageIndexerGUI.sort_images_window import ScanImagesWindow

class LaunchWindow(ctk.CTkFrame):
    def __init__(self, parent, *args, **kwargs):
        ctk.CTkFrame.__init__(self, parent, *args, **kwargs)
        self.parent = parent

        self.generate_button = ctk.CTkButton(master=self, text='Generate QR Codes', command=self.click_generate_button)
        self.generate_button.pack(fill='both', side='top', expand=True)
        self.sort_button = ctk.CTkButton(master=self, text='Sort Images', command=self.click_sort_button)
        self.sort_button.pack(fill='both', side='bottom', expand=True)

        self.pack(side='top', fill='both', expand=True)

    def click_generate_button(self):
        generate_qr_window.GenerateQRWindow(self)

    def click_sort_button(self):
        ScanImagesWindow(self)


import customtkinter as ctk

import tkinter as tk
from tkinter import filedialog, messagebox
from typing import Dict

from PIL import Image

import io

import fitz
from qrImageIndexer.qr_generator import load_lines
from qrImageIndexer.write_pdf_fpf2 import FPDF
from qrImageIndexer.generate_qr_wrapper import generate_qr_pdf

from qrImageIndexerGUI.image_grid import ImageGrid

FRAME_PAD_PX = 10

SAMPLE_TEXT = """Line 1
\tLine 1 indented
\t\tLine 1 twice indented
Line 2
Line 3
\tLine 3 indented"""

class PDFViewer(ImageGrid):
    def __init__(self, master, *args, **kwargs):
        ImageGrid.__init__(self, master, 'FILL', *args, **kwargs)
    
    def show(self, pdf_file, stream=None):
        self.clear_grid()
        pdf = fitz.Document(pdf_file, stream=stream) # open the PDF file
        for i, page in enumerate(pdf):
            pix = page.get_pixmap(dpi=300)
            pix1 = fitz.Pixmap(pix, 0) if pix.alpha else pix
            photo = Image.open(io.BytesIO(pix1.tobytes('png')))
            # insert into the text box
            self.add_plain_image(photo, i)
        self.rebuild_grid()


class OptionsFrame(ctk.CTkFrame):
    def __init__(self, parent, *args, **kwargs):
        ctk.CTkFrame.__init__(self, parent, *args, **kwargs)
        self.parent = parent

        self.sort_sliceable_chk_var = tk.BooleanVar()
        self.sort_sliceable_chk_var.set(True)
        self.sort_sliceable_chk = ctk.CTkCheckBox(self, text='Sort PDF to be sliceable (cut through page for ordered slips)', 
                                        variable=self.sort_sliceable_chk_var, onvalue=True,
                                        offvalue=False)

        self.sort_sliceable_chk.pack(side=tk.TOP, fill=tk.BOTH)

        self.qr_for_headings_chk_var = tk.BooleanVar()
        self.qr_for_headings_chk_var.set(True)
        self.qr_for_headings_chk = ctk.CTkCheckBox(self, text='Include QR codes for heading all lines (not just last level)', 
                                        variable=self.qr_for_headings_chk_var, onvalue=True,
                                        offvalue=False)

        self.qr_for_headings_chk.pack(side=tk.TOP, fill=tk.BOTH)

        self.repeat_headings_chk_var = tk.BooleanVar()
        self.repeat_headings_chk_var.set(True)
        self.repeat_headings_chk = ctk.CTkCheckBox(self, text='Repeat heading text on every line', 
                                        variable=self.repeat_headings_chk_var, onvalue=True,
                                        offvalue=False)

        self.repeat_headings_chk.pack(side=tk.TOP, fill=tk.BOTH)

        self.use_prefix_chk_var = tk.BooleanVar()
        self.use_prefix_chk_var.set(True)
        self.use_prefix_chk = ctk.CTkCheckBox(self, text='Include prefix in QR code text', 
                                        variable=self.use_prefix_chk_var, onvalue=True,
                                        offvalue=False, command=self.prefix_toggle)

        self.use_prefix_chk.pack(side=tk.TOP, fill=tk.BOTH)

        self.prefix_frame = ctk.CTkFrame(self)
        self.prefix_frame.pack(side=tk.TOP, fill=tk.BOTH)

        self.prefix_label = ctk.CTkLabel(self.prefix_frame, text="Prefix:")
        self.prefix_label.pack(side=tk.LEFT, fill=tk.BOTH)

        self.prefix_input = ctk.CTkEntry(self.prefix_frame)
        self.prefix_input.insert(0, r"{image}")
        self.prefix_input.pack(side=tk.RIGHT, fill=tk.BOTH, expand=True)




    def get_check_states(self) -> Dict[str, bool]:
        return {
            'sliceable' : self.sort_sliceable_chk_var.get(),
            'qr_headings' : self.qr_for_headings_chk_var.get(),
            'repeat_headings' : self.repeat_headings_chk_var.get(),
            'use_prefix' : self.use_prefix_chk_var.get(),
        }

    def get_prefix(self) -> str:
        return self.prefix_input.get()
    
    def prefix_toggle(self):
        if not self.use_prefix_chk_var.get():
            self.prefix_frame.pack_forget()
        else:
            self.prefix_frame.pack(side=tk.TOP, fill=tk.BOTH)




class GenerateQRWindow(ctk.CTkToplevel):
    def __init__(self, master, *args, **kwargs):
        ctk.CTkToplevel.__init__(self, master, *args, **kwargs)
        self.geometry("1200x800")
        self.title("Generate QR Codes")

        self.top_frame = ctk.CTkFrame(self)
        self.top_frame.pack(fill='x', side=tk.TOP)

        self.save_button = ctk.CTkButton(self.top_frame, text='Save PDF to File', command=self.save_pdf)
        self.save_button.pack(fill='x', side=tk.LEFT, expand=True)

        self.generate_sample_button = ctk.CTkButton(self.top_frame, text='Update PDF Sample', command=self.update_pdf_sample)
        self.generate_sample_button.pack(fill='x', side=tk.RIGHT, expand=True)

        self.left_frame = ctk.CTkFrame(self)
        self.left_frame.pack(fill=tk.BOTH, expand=True, side=tk.LEFT)

        self.right_frame = ctk.CTkFrame(self)
        self.right_frame.pack(fill=tk.BOTH, expand=True, side=tk.RIGHT)

        self.opt_frame = OptionsFrame(self.left_frame)
        self.opt_frame.pack(fill=tk.BOTH, side=tk.TOP)

        self.enter_txt = ctk.CTkTextbox(self.left_frame)
        self.enter_txt.insert("1.0", text=SAMPLE_TEXT)
        self.enter_txt.pack(fill=tk.BOTH, expand=True, side='bottom',
                            padx=FRAME_PAD_PX, pady=FRAME_PAD_PX)

        self.doc_viewer = PDFViewer(self.right_frame)#,
            # bg=self.cget('fg_color')[1] if ctk.get_appearance_mode() == 'Dark' else self.cget('fg_color')[0])
        self.doc_viewer.pack(fill=tk.BOTH, expand=True, side=tk.TOP,
                            padx=FRAME_PAD_PX, pady=FRAME_PAD_PX)

        self.update_pdf_sample() #Update the PDF here so that it will contain the sample text and use all the default settings

    def save_pdf(self):
        file = filedialog.asksaveasfile(mode='wb', confirmoverwrite=True, defaultextension='.pdf',
                                        filetypes=[['PDF Files', '*.pdf']])
        pdf = self.generate_pdf()
        pdf.output(file)
        file.close()
        messagebox.showinfo("File Saved", "File successfully saved.")

    def update_pdf_sample(self):
        self.doc_viewer.show(None, self.generate_pdf().output())

    def generate_pdf(self) -> FPDF:
        check_states = self.opt_frame.get_check_states()
        text = self.enter_txt.get("1.0", tk.END)
        text = text.split('\n')
        data = load_lines(text)
        prefix = ''
        if check_states['use_prefix']:
            prefix = self.opt_frame.get_prefix()
        return generate_qr_pdf(data, check_states['qr_headings'], check_states['repeat_headings'],
                                check_states['sliceable'], prefix)


from typing import Dict, List, Union

import tkinter as tk
import customtkinter as ctk
import natsort
from PIL import ImageTk, Image

DEFAULT_COLUMN_WIDTH = 400

class PlainImagePresent(ctk.CTkFrame):
    
    def __init__(self, master, image: Image.Image, *args, **kwargs):
        ctk.CTkFrame.__init__(self, master, *args, **kwargs)
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=1)
        
        self.original_image: Image.Image = image
        self.image = ImageTk.PhotoImage(self.original_image)
        self.image_canv = ctk.CTkCanvas(self, bd=0, highlightthickness=0)
        self.image_canv.create_image(0, 0, image=self.image, anchor=tk.NW, tags='IMG')
        # self.image_canv.pack(side=tk.TOP, fill=tk.BOTH)
        self.image_canv.grid(row=0, sticky=tk.NSEW)

        self.bind('<Configure>', self.resize)
        # self.image_lbl = tk.Label(self, image=self.image)
        # self.image_lbl.pack(side=tk.TOP, fill=tk.BOTH)

    def resize_by_width(self, width):
        image_ar = self.original_image.size[1] / self.original_image.size[0]
        adjusted_size = (width, int(width * image_ar))
        resized = self.original_image.copy()
        resized.thumbnail(adjusted_size,Image.ANTIALIAS)
        background = Image.new('RGBA', adjusted_size, (255, 255, 255, 0))
        background.paste(
            resized, (int((adjusted_size[0] - resized.size[0]) / 2), int((adjusted_size[1] - resized.size[1]) / 2))
        )
        self.image = ImageTk.PhotoImage(background)
        self.image_canv.delete('IMG')
        self.image_canv.create_image(0, 0, image=self.image, anchor=tk.NW, tags='IMG')
        self.image_canv.configure(width=adjusted_size[0], height=adjusted_size[1])

    def resize(self, event):
        self.resize_by_width(event.width)


class KeyImagePresent(ctk.CTkFrame):
    
    def __init__(self, master, image_path : str, extracted_path : str, *args, **kwargs):
        ctk.CTkFrame.__init__(self, master, *args, **kwargs)
        self.columnconfigure(0, weight=1)
        self.rowconfigure(0, weight=0)
        self.rowconfigure(1, weight=1)
        self.rowconfigure(2, weight=0)
        
        self.original_image = Image.open(image_path)
        self.image = ImageTk.PhotoImage(self.original_image)
        self.image_canv = ctk.CTkCanvas(self, bd=0, highlightthickness=0)
        self.image_canv.create_image(0, 0, image=self.image, anchor=tk.NW, tags='IMG')
        # self.image_canv.pack(side=tk.TOP, fill=tk.BOTH)
        self.image_canv.grid(row=0, sticky=tk.NSEW)

        self.bind('<Configure>', self.resize)
        # self.image_lbl = tk.Label(self, image=self.image)
        # self.image_lbl.pack(side=tk.TOP, fill=tk.BOTH)

        self.path = extracted_path
        self.path_lbl = ctk.CTkLabel(self, text=self.path, justify=tk.CENTER)
        # self.path_lbl.pack(side=tk.BOTTOM, fill=tk.BOTH)
        self.path_lbl.grid(row=1, sticky=tk.NSEW)

        self.image_path = image_path
        self.image_path_lbl = ctk.CTkLabel(self, text=self.image_path, justify=tk.CENTER)
        # self.path_lbl.pack(side=tk.BOTTOM, fill=tk.BOTH)
        self.image_path_lbl.grid(row=2, sticky=tk.NSEW)

    def resize_by_width(self, width):
        image_ar = self.original_image.size[1] / self.original_image.size[0]
        adjusted_size = (width, int(width * image_ar))
        resized = self.original_image.copy()
        resized.thumbnail(adjusted_size,Image.ANTIALIAS)
        background = Image.new('RGBA', adjusted_size, (255, 255, 255, 0))
        background.paste(
            resized, (int((adjusted_size[0] - resized.size[0]) / 2), int((adjusted_size[1] - resized.size[1]) / 2))
        )
        self.image = ImageTk.PhotoImage(background)
        self.image_canv.delete('IMG')
        self.image_canv.create_image(0, 0, image=self.image, anchor=tk.NW, tags='IMG')
        self.image_canv.configure(width=adjusted_size[0], height=adjusted_size[1])

    def resize(self, event):
        self.resize_by_width(event.width)


class ImageGrid(ctk.CTkFrame):
    def __init__(self, master, column_width: Union[int, str] = DEFAULT_COLUMN_WIDTH, *args, **kwargs):
        ctk.CTkFrame.__init__(self, master, *args, **kwargs)
        self.images : Dict[str, Union[PlainImagePresent, KeyImagePresent]] = {}
        self._column_width_setting = column_width

        self.vscrollbar = ctk.CTkScrollbar(self)
        self.scrolled_canvas = ctk.CTkCanvas(self, bd=0, highlightthickness=0,
                            bg=self.cget('fg_color')[1] if ctk.get_appearance_mode() == 'Dark' else self.cget('fg_color')[0])
        self.frame = ctk.CTkFrame(self.scrolled_canvas)

        self.vscrollbar.pack(side=tk.RIGHT, fill=tk.Y)
        self.scrolled_canvas.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
        self.scrolled_canvas.create_window((0,0), window=self.frame, anchor=tk.NW,
                                            tags='self.frame')

        self.vscrollbar.configure(command=self.scrolled_canvas.yview)
        self.scrolled_canvas.configure(yscrollcommand=self.vscrollbar.set)

        self.scrolled_canvas.xview_moveto(0)
        self.scrolled_canvas.yview_moveto(0)

        self.bind('<Configure>', self.resize_canvas)
        self.frame.bind('<Configure>', self.on_frame_configure)

        self.used_cols = 0

    @property
    def column_width(self) -> int:
        if isinstance(self._column_width_setting, int):
            return self._column_width_setting
        elif isinstance(self._column_width_setting, str) and self._column_width_setting == 'FILL':
            return self.winfo_width()
        else:
            raise ValueError(f'Invalid setting for column width: {self._column_width_setting}')

    @property
    def is_single_full_width(self) -> bool:
        if self._column_width_setting == 'FILL':
            return True
        else:
            return False

    @property
    def items_sorted(self):
        sorted_keys = natsort.natsorted(self.images.keys())
        return [self.images[x] for x in sorted_keys]

    def clear_grid(self):
        for key in self.images:
            self.images[key].destroy()
        self.images.clear()

    def on_frame_configure(self, event):
        self.scrolled_canvas.configure(scrollregion=self.scrolled_canvas.bbox(tk.ALL))

    def resize_canvas(self, event):
        """
            Resize has occurred, rebuild the grid based on the resize width/height
        """
        size = (event.width, event.height)
        self.rebuild_grid(size)

    def rebuild_grid(self, size = None):
        """
            Rebuild the grid based on the provided size
        """
        if size is None:
            size = (self.frame.winfo_width(), self.frame.winfo_height())
        columns = size[0]//self.column_width
        
        if columns == 0:
            columns = 1

        # Track the highest number of columns used so that they can
        # all be reset to empty if needed
        if columns > self.used_cols:
            self.used_cols = columns

        for i in range(columns):
            self.frame.columnconfigure(i, weight=1)

        if columns < self.used_cols:
            for i in range(columns, self.used_cols):
                self.frame.columnconfigure(i, weight=0)

        gridified_items = ImageGrid.gridify_items(self.items_sorted, columns)
        for irow, row in enumerate(gridified_items):
            item : ctk.CTkFrame
            for icol, item in enumerate(row):
                item.grid(column=icol, row=irow, sticky=tk.NSEW)
                if self.is_single_full_width: item.resize_by_width(self.column_width)

    @staticmethod
    def gridify_items(items : List, columns : int) -> List[List[Union[PlainImagePresent, KeyImagePresent]]]:
        """
            Takes a single dimensional list and splits it into a list of
            lists that contains the specified number of columns
        """
        result = []
        current_row = None
        for index, item in enumerate(items):
            column = index % columns
            if column == 0:
                current_row = []
                result.append(current_row)
            current_row.append(item)
        
        return result

    def add_key_image(self, image_path : str, qr_value : str) -> None:
        image = KeyImagePresent(self.frame, image_path, qr_value)
        self.images[image_path] = image

    def add_plain_image(self, image_in: tk.PhotoImage, index: int) -> None:
        image = PlainImagePresent(self.frame, image_in)
        self.images[str(index)] = image



from qrImageIndexerGUI import sort_images_window
import pytest
import tkinter as tk
import customtkinter as ctk
from unittest.mock import MagicMock, call, Mock

def test_gridify_items_multiline():
    input = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    columns = 4
    expected = [
        [1, 2, 3, 4],
        [5, 6, 7, 8],
        [9]
    ]

    assert expected == sort_images_window.ImageGrid.gridify_items(input, columns)

def test_gridify_items_oneline():
    input = [1, 2, 3, 4, 5, 6, 7]
    columns = 8
    expected = [
        [1, 2, 3, 4, 5, 6, 7]
    ]

    assert expected == sort_images_window.ImageGrid.gridify_items(input, columns)

def test_gridify_items_noitems():
    input = []
    columns = 5
    expected = [

    ]
    assert expected == sort_images_window.ImageGrid.gridify_items(input, columns)

@pytest.fixture
def options_frame() -> sort_images_window.ScanOptionsFrame:
    app = tk.Tk()
    frame = sort_images_window.ScanOptionsFrame(app)
    yield frame
    app.destroy()

def test_get_prefix_checked(options_frame : sort_images_window.ScanOptionsFrame):
    options_frame.has_prefix_chk.select()
    options_frame.prefix_input.delete(0, tk.END)
    options_frame.prefix_input.insert(0, 'prefix')
    assert 'prefix' == options_frame.get_prefix()

def test_get_prefix_unchecked(options_frame : sort_images_window.ScanOptionsFrame):
    options_frame.has_prefix_chk.deselect()
    options_frame.prefix_input.delete(0, tk.END)
    options_frame.prefix_input.insert(0, 'prefix')
    assert '' == options_frame.get_prefix()

def test_sorted_keys_in_order():
    app = ctk.CTk()
    image_grid = sort_images_window.ImageGrid(app)
    image_grid.images['a'] = 'TEST A'
    image_grid.images['b'] = 'TEST B'
    image_grid.images['c'] = 'TEST C'
    image_grid.images['d'] = 'TEST D'

    expected = [
        'TEST A',
        'TEST B',
        'TEST C',
        'TEST D',
        ]
    
    assert expected == image_grid.items_sorted



def test_sorted_keys_out_of_order():
    app = ctk.CTk()
    image_grid = sort_images_window.ImageGrid(app)
    image_grid.images['a'] = 'TEST A'
    image_grid.images['c'] = 'TEST C'
    image_grid.images['b'] = 'TEST B'
    image_grid.images['d'] = 'TEST D'

    expected = [
        'TEST A',
        'TEST B',
        'TEST C',
        'TEST D',
        ]
    
    assert expected == image_grid.items_sorted

def is_image_side_effect(image_path):
    if 'non-image' in image_path:
        return (False, image_path)
    else:
        return (True, image_path)

def get_qr_mt_side_effect(image_path, string_header, binarization):
    if not binarization:
        assert False
    if 'qr' in image_path:
        qr = image_path[image_path.find('qr')+2:]
        if string_header in qr:
            return image_path, qr[len(string_header):]
    return image_path, None

@pytest.fixture
def mocked_image_sort_functions(mocker):
    is_image = mocker.patch('qrImageIndexerGUI.sort_images_window.photo_sorter.check_if_image',
                            side_effect=is_image_side_effect)

    # get_qr_mt = mocker.patch('qrImageIndexerGUI.sort_images_window.photo_sorter.get_qr_mt',
    #                         side_effect=get_qr_mt_side_effect)
    
    get_qr_mt = None

    #Due to the use of multiprocessing, this function could not easily be mocked as MagicMock could not be pickled
    #so for now just going to directly replace the function.
    sort_images_window.photo_sorter.get_qr_mt = get_qr_mt_side_effect

    return (is_image, get_qr_mt)



def test_image_scan_no_qr(mocked_image_sort_functions):
    is_image : Mock
    get_qr_mt : Mock
    (is_image, get_qr_mt) = mocked_image_sort_functions
    files = [
        'image1',
        'image2',
        'image3'
    ]
    thread = sort_images_window.ImageScan(files, '')
    thread.start()
    thread.join()

    assert 0 == len(thread.results)
    assert 3 == len(thread.files)
    assert 3 == len(thread.image_files)
    assert 0 == len(thread.non_image_files)

    is_image.assert_has_calls([call(x) for x in files])
    # get_qr_mt.assert_has_calls([call(x, '', True) for x in files])


def test_image_scan_1_qr(mocked_image_sort_functions):
    is_image : Mock
    get_qr_mt : Mock
    (is_image, get_qr_mt) = mocked_image_sort_functions
    files = [
        'image1',
        'image2qr123',
        'image3'
    ]
    thread = sort_images_window.ImageScan(files, '')
    thread.start()
    thread.join()

    assert 1 == len(thread.results)
    assert '123' == thread.results['image2qr123']
    assert 3 == len(thread.files)
    assert 3 == len(thread.image_files)
    assert 0 == len(thread.non_image_files)

    is_image.assert_has_calls([call(x) for x in files])

def test_image_scan_1_non_image(mocked_image_sort_functions):
    is_image : Mock
    get_qr_mt : Mock
    (is_image, get_qr_mt) = mocked_image_sort_functions
    files = [
        'image1',
        'image2qr123',
        'image3non-image'
    ]
    thread = sort_images_window.ImageScan(files, '')
    thread.start()
    thread.join()

    assert 1 == len(thread.results)
    assert '123' == thread.results['image2qr123']
    assert 3 == len(thread.files)
    assert 2 == len(thread.image_files)
    assert 1 == len(thread.non_image_files)
    assert thread.non_image_files == ['image3non-image']

    is_image.assert_has_calls([call(x) for x in files])

def test_sort_dict_function():
    input = {
        'a':123,
        'b':456,
        'c':789
    }

    expect_keys = ['a', 'b', 'c']
    
    assert expect_keys == list(sort_images_window.ScanImagesWindow.sort_results_dict(input).keys())

def test_sort_dict_function_unergdered():
    input = {
        'a':123,
        'c':789,
        'b':456
    }

    expect_keys = ['a', 'b', 'c']
    
    assert expect_keys == list(sort_images_window.ScanImagesWindow.sort_results_dict(input).keys())


import pytest
import tkinter as tk
import customtkinter as ctk
from qrImageIndexerGUI import generate_qr_window

# Testing the behaviour of the checkboxes. Probably not the best practice, but wanted to make sure that
# they are representing the checked state of the appropriate checkboxes

@pytest.fixture
def options_frame() -> generate_qr_window.OptionsFrame:
    app = ctk.CTk()
    frame = generate_qr_window.OptionsFrame(app)
    yield frame
    app.destroy()


def test_sliceable_checkbox_check(options_frame : generate_qr_window.OptionsFrame):
    options_frame.sort_sliceable_chk.select()
    assert options_frame.get_check_states()['sliceable']

def test_sliceable_checkbox_uncheck(options_frame : generate_qr_window.OptionsFrame):
    options_frame.sort_sliceable_chk.deselect()
    assert not options_frame.get_check_states()['sliceable']

def test_qr_headings_checkbox_check(options_frame : generate_qr_window.OptionsFrame):
    options_frame.qr_for_headings_chk.select()
    assert options_frame.get_check_states()['qr_headings']

def test_qr_headings_checkbox_uncheck(options_frame : generate_qr_window.OptionsFrame):
    options_frame.qr_for_headings_chk.deselect()
    assert not options_frame.get_check_states()['qr_headings']

def test_repeat_headings_checkbox_check(options_frame : generate_qr_window.OptionsFrame):
    options_frame.repeat_headings_chk.select()
    assert options_frame.get_check_states()['repeat_headings']

def test_repeat_headings_checkbox_uncheck(options_frame : generate_qr_window.OptionsFrame):
    options_frame.repeat_headings_chk.deselect()
    assert not options_frame.get_check_states()['repeat_headings']

def test_use_prefix_checkbox_check(options_frame : generate_qr_window.OptionsFrame):
    options_frame.use_prefix_chk.select()
    assert options_frame.get_check_states()['use_prefix']

def test_use_prefix_checkbox_uncheck(options_frame : generate_qr_window.OptionsFrame):
    options_frame.use_prefix_chk.deselect()
    assert not options_frame.get_check_states()['use_prefix']

def test_get_prefix_option(options_frame : generate_qr_window.OptionsFrame):
    options_frame.prefix_input.delete(0, tk.END)
    options_frame.prefix_input.insert(0, 'prefix')
    assert options_frame.get_prefix() == 'prefix'

@pytest.fixture
def mocked_qr_generate_functions(mocker):

    get_check_states = mocker.patch('qrImageIndexerGUI.generate_qr_window.OptionsFrame.get_check_states')

    get_prefix = mocker.patch('qrImageIndexerGUI.generate_qr_window.OptionsFrame.get_prefix')

    load_lines = mocker.patch('qrImageIndexerGUI.generate_qr_window.load_lines', return_value=[
                                ['Line 1', ''],
                                ['', 'Line 1 Indent']
                            ])

    generate_qr_pdf = mocker.patch('qrImageIndexerGUI.generate_qr_window.generate_qr_pdf', 
                                return_value='pdf_document')
    
    return (get_check_states, get_prefix, load_lines, generate_qr_pdf)


@pytest.fixture
def generate_qr(mocker) -> generate_qr_window.GenerateQRWindow:
    mocker.patch('qrImageIndexerGUI.generate_qr_window.GenerateQRWindow.update_pdf_sample')
    app = tk.Tk()
    frame = generate_qr_window.GenerateQRWindow(app)
    frame.enter_txt.delete("1.0", tk.END)
    frame.enter_txt.insert("1.0", "Input Text")
    yield frame
    app.destroy()

def test_generate_pdf_all_opts(mocked_qr_generate_functions, generate_qr : generate_qr_window.GenerateQRWindow):
    (get_check_states, get_prefix, load_lines, generate_qr_pdf) = mocked_qr_generate_functions
    get_check_states.return_value = {
            'sliceable' : True,
            'qr_headings' : True,
            'repeat_headings' : True,
            'use_prefix' : True,
    }

    get_prefix.return_value = r'{prefix}'

    result = generate_qr.generate_pdf()
    get_check_states.assert_called_once_with()
    load_lines.assert_called_once_with(["Input Text", ""])
    generate_qr_pdf.assert_called_once_with(load_lines.return_value, True,
                                            True, True, get_prefix.return_value)
    assert result == generate_qr_pdf.return_value

def test_generate_pdf_no_slice(mocked_qr_generate_functions, generate_qr : generate_qr_window.GenerateQRWindow):
    (get_check_states, get_prefix, load_lines, generate_qr_pdf) = mocked_qr_generate_functions
    get_check_states.return_value = {
            'sliceable' : False,
            'qr_headings' : True,
            'repeat_headings' : True,
            'use_prefix' : True,
    }

    get_prefix.return_value = r'{prefix}'

    result = generate_qr.generate_pdf()
    get_check_states.assert_called_once_with()
    load_lines.assert_called_once_with(["Input Text", ""])
    generate_qr_pdf.assert_called_once_with(load_lines.return_value, True,
                                            True, False, get_prefix.return_value)
    assert result == generate_qr_pdf.return_value

def test_generate_pdf_no_qr_heading(mocked_qr_generate_functions, generate_qr : generate_qr_window.GenerateQRWindow):
    (get_check_states, get_prefix, load_lines, generate_qr_pdf) = mocked_qr_generate_functions
    get_check_states.return_value = {
            'sliceable' : True,
            'qr_headings' : False,
            'repeat_headings' : True,
            'use_prefix' : True,
    }

    get_prefix.return_value = r'{prefix}'

    result = generate_qr.generate_pdf()
    get_check_states.assert_called_once_with()
    load_lines.assert_called_once_with(["Input Text", ""])
    generate_qr_pdf.assert_called_once_with(load_lines.return_value, False,
                                            True, True, get_prefix.return_value)
    assert result == generate_qr_pdf.return_value

def test_generate_pdf_no_repeat_heading(mocked_qr_generate_functions, generate_qr : generate_qr_window.GenerateQRWindow):
    (get_check_states, get_prefix, load_lines, generate_qr_pdf) = mocked_qr_generate_functions
    get_check_states.return_value = {
            'sliceable' : True,
            'qr_headings' : True,
            'repeat_headings' : False,
            'use_prefix' : True,
    }

    get_prefix.return_value = r'{prefix}'

    result = generate_qr.generate_pdf()
    get_check_states.assert_called_once_with()
    load_lines.assert_called_once_with(["Input Text", ""])
    generate_qr_pdf.assert_called_once_with(load_lines.return_value, True,
                                            False, True, get_prefix.return_value)
    assert result == generate_qr_pdf.return_value

def test_generate_pdf_no_prefix(mocked_qr_generate_functions, generate_qr : generate_qr_window.GenerateQRWindow):
    (get_check_states, get_prefix, load_lines, generate_qr_pdf) = mocked_qr_generate_functions
    get_check_states.return_value = {
            'sliceable' : True,
            'qr_headings' : True,
            'repeat_headings' : True,
            'use_prefix' : False,
    }

    get_prefix.return_value = r'{prefix}'

    result = generate_qr.generate_pdf()
    get_check_states.assert_called_once_with()
    load_lines.assert_called_once_with(["Input Text", ""])
    generate_qr_pdf.assert_called_once_with(load_lines.return_value, True,
                                            True, True, '')
    assert result == generate_qr_pdf.return_value


from qrImageIndexerGUI import launch_window
import customtkinter as ctk

def main():
    print("launched")
    ctk.set_appearance_mode("System")
    ctk.set_default_color_theme("blue")
    app = ctk.CTk(className="qrImageIndexerGUI")
    launch_window.LaunchWindow(app)
    app.title('QR Image Indexer GUI')
    app.geometry("300x75")
    app.resizable(width=False, height=False)
    app.mainloop()
    

if __name__ == '__main__':
    main()


